@@ -13,64 +13,38 @@
 #   probit use probit transform rather than logitic to obtain outcome probabilities
 #   verbose print information about algorithm    
 
+	patterns <- as.matrix(patterns)
+	mode(patterns) <- "double"
+
 	nrepeats <- dim(patterns)[2]/blocksize
 	nlevel1 <- dim(patterns)[2]
 	nlevel2 <- length(freq)
 
 	if (verbose) print("fit.random.randomLCA")
 
-		# first rearrange the patterns
-		npatterns <- 1-patterns
-		# if an NA setting to zero in both patterns excludes from calculation
-		myoutcomes <- ifelse(is.na(as.matrix(patterns)),0,as.matrix(patterns))
-		mynoutcomes <- ifelse(is.na(as.matrix(npatterns)),0,as.matrix(npatterns))
-
-		newmyoutcomes <- NULL
-		for (i in 1:length(gh[,1])) newmyoutcomes <- rbind(newmyoutcomes,myoutcomes)
-		newmynoutcomes <- NULL
-		for (i in 1:length(gh[,1])) newmynoutcomes <- rbind(newmynoutcomes,mynoutcomes)
-
-		calclikelihood <- function(classx,outcomex,lambdacoef,momentdata,gh,newmyoutcomes,newmynoutcomes) {
+		calclikelihood <- function(classx,outcomex,lambdacoef,momentdata,gh,patterns,calcfitted=FALSE) {
 			#starttime <- proc.time()
 			# turn classx into actual probabilities
 			classp2 <- c(0,classx)       
 			classp2 <- exp(classp2)/sum(exp(classp2))
-			# calculate the likelihood over all classes and level 2 units
-			ltotal <- NULL
-			#ll <- matrix(rep(NA,nclass*nlevel2),ncol=nclass)
-			ll <- NULL
-			#ll2 <- matrix(rep(NA,length(gh[,1])*nlevel2),nrow=nlevel2)
-			level2p <- momentdata[,1]+momentdata[,2] %o% gh[,1]
-			level2w <- log(sqrt(2*pi))+log(momentdata[,2])+
-				matrix(rep(gh[,1]^2/2+log(gh[,2]),each=nlevel2),nrow=nlevel2)+
-				matrix(dnorm(as.vector(level2p),log=TRUE),nrow=nlevel2)
+			ill <- matrix(rep(NA,nclass*length(freq)),ncol=nclass)
 			for (iclass in 1:nclass) {
-				# outcome data
-				#myoutcomex <- outcomex[iclass,]
-				# now start calculating for each level 2 quadrature point
-				myoutcomex2 <- t(outcomex[iclass,]+t(as.vector(level2p) %o% rep(lambdacoef,nrepeats)))
-				if (probit) {
-					lmyoutcomep <- pnorm(myoutcomex2,log.p=TRUE)
-					nlmyoutcomep <- pnorm(-myoutcomex2,log.p=TRUE)
-				}
-				else {
-					lmyoutcomep <- -log(1+exp(-myoutcomex2))
-					nlmyoutcomep <- -log(1+exp(myoutcomex2))
-				}
-				ll2 <- matrix(rowSums(newmyoutcomes*lmyoutcomep+newmynoutcomes*nlmyoutcomep),ncol=length(gh[,1]))
-				# final level 2 likelihoods
-				ll <- cbind(ll,log(rowSums(exp(ll2+level2w),na.rm=TRUE)))
+				ill[,iclass] <- .Call("bernoulliprobrandom",patterns,outcomex[iclass,],lambdacoef,
+					gh,momentdata,probit)*classp2[iclass]
+				#browser()
 			}
-			ill <- t(t(exp(ll))*classp2)
 			ill2 <- rowSums(ill,na.rm=TRUE)
 			ll <- sum(log(ill2)*freq,na.rm=TRUE)
-			fitted <- ill2*sum(ifelse(apply(patterns,1,function(x) any(is.na(x))),0,freq))*
-				ifelse(apply(patterns,1,function(x) any(is.na(x))),NA,1)
-			classprob <- ill/ill2
-			return(list(logl=ll,fitted=fitted,classprob=classprob))
+			#print(ll)
+			if (calcfitted) {
+				fitted <- ill2*sum(ifelse(apply(patterns,1,function(x) any(is.na(x))),0,freq))*
+					ifelse(apply(patterns,1,function(x) any(is.na(x))),NA,1)
+				classprob <- ill/ill2
+				return(list(logl=ll,fitted=fitted,classprob=classprob))
+			} else return(list(logl=ll))
 		}  # end of calclikelihood
 
-	calcrandom <- function(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes) {
+	calcrandom <- function(classx,outcomex,lambdacoef,momentdata) {
 
 		classx <- c(0,classx)       
 		classp <- exp(classx)/sum(exp(classx))
@@ -100,23 +74,23 @@
 		return(betas)
 	}
 	
-	adaptivefit <- function(classx,outcomex,lambdacoef,momentdata,calcSE,gh,newmyoutcomes,newmynoutcomes) {
+	adaptivefit <- function(classx,outcomex,lambdacoef,calcSE,momentdata,gh,patterns) {
 	
 	
 		fitparams <- function(classx,outcomex,lambdacoef,
-			momentdata,calcSE,gh,newmyoutcomes,newmynoutcomes,noiterations=10) {
-			calcllfornlm <- function(params,momentdata,gh,newmyoutcomes,newmynoutcomes) {
+			momentdata,calcSE,gh,patterns,noiterations=10) {
+			calcllfornlm <- function(params,momentdata,gh,patterns) {
 				oneiteration <- calclikelihood(if (nclass==1) NULL else params[1:(nclass-1)],
 					matrix(params[nclass:(nclass+nlevel1*nclass-1)],nrow=nclass),
 					params[(nlevel1*nclass+nclass):(nlevel1*nclass+nclass+blocksize-1)],
-					momentdata,gh,newmyoutcomes,newmynoutcomes)
+					momentdata,gh,patterns)
 				ll <- -oneiteration$logl
 				ll
 			}
 			
 			nlm1 <- nlm(calcllfornlm, c(classx, as.vector(outcomex), lambdacoef), iterlim = noiterations,
 				print.level=ifelse(verbose,2,0),hessian=calcSE,
-				check.analyticals = FALSE,momentdata=momentdata,gh=gh,newmyoutcomes=newmyoutcomes,newmynoutcomes=newmynoutcomes)
+				check.analyticals = FALSE,momentdata=momentdata,gh=gh,patterns=patterns)
 			return(list(logl=-nlm1$minimum,
 				classx=(if (nclass==1) NULL else nlm1$estimate[1:(nclass-1)]),
 				outcomex=matrix(nlm1$estimate[nclass:(nclass+nlevel1*nclass-1)],nrow=nclass),
@@ -125,14 +99,12 @@
 		}
 	
 
-		oneiteration <- calclikelihood(classx,outcomex,lambdacoef,
-			momentdata,gh,newmyoutcomes,newmynoutcomes)
+		oneiteration <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns)
 		currll <- oneiteration$logl
 		if (verbose) cat('Initial ll',currll,"\n")
 	# shift the quadrature points for the first time
-		momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes)
-		oneiteration <- calclikelihood(classx,outcomex,lambdacoef,
-		momentdata,gh,newmyoutcomes,newmynoutcomes)
+		momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata)
+		oneiteration <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns)
 		currll <- oneiteration$logl
 		if (verbose) cat("current ll",currll,"\n")		
 		
@@ -140,17 +112,15 @@
 		prevll <- -Inf
         while(adaptive) {
 			# need to do an optimisation on the other parameters
-			fitresults <- fitparams(classx,outcomex,lambdacoef,
-				momentdata,FALSE,gh,newmyoutcomes,newmynoutcomes)
+			fitresults <- fitparams(classx,outcomex,lambdacoef,momentdata,FALSE,gh,patterns)
 			currll <- fitresults$logl
 			outcomex <- fitresults$outcomex
 			classx <- fitresults$classx
 			lambdacoef <- fitresults$lambdacoef
 			if (verbose) cat("current ll from optimisation",currll,"\n")		
 			# shift the quadrature points again
-			momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes)
-			oneiteration <- calclikelihood(classx,outcomex,lambdacoef,
-			momentdata,gh,newmyoutcomes,newmynoutcomes)
+			momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata)
+			oneiteration <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns)
 			# check if moving quadrature points has changed likelihood
         	adaptive <- (abs((oneiteration$logl-currll)/oneiteration$logl)>1.0e-7) ||
         		(abs((oneiteration$logl-prevll)/oneiteration$logl)>1.0e-7)
@@ -159,8 +129,7 @@
         	if ((prevll-currll)/abs(currll) > 1.0e-4) stop("divergence - increase quadrature points")
         	prevll <- currll
 		}
-		fitresults <- fitparams(classx,outcomex,lambdacoef,
-				momentdata,calcSE,gh,newmyoutcomes,newmynoutcomes,noiterations=500)
+		fitresults <- fitparams(classx,outcomex,lambdacoef,momentdata,calcSE,gh,patterns,noiterations=500)
 		return(list(nlm=fitresults$nlm,momentdata=momentdata))
 	} # end adaptivefit
 
@@ -194,21 +163,19 @@
  		repeat {
 			if (verbose) cat('trying lambdacoef ',testlambdacoef,"\n")
 			lambdacoef <- rep(testlambdacoef,blocksize)
-			oneiteration <- calclikelihood(classx,outcomex,lambdacoef,
-				momentdata,gh,newmyoutcomes,newmynoutcomes)
+			oneiteration <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns)
 			currll <- oneiteration$logl
 			if (verbose) cat('Initial ll',currll,"\n")
 			lastll <- 2*currll
 		# shift the quadrature points for the first time
 			while (abs((lastll-currll)/lastll)>1.0e-6) {
 				lastll <- currll
-				momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes)
-				oneiteration <- calclikelihood(classx,outcomex,lambdacoef,
-				momentdata,gh,newmyoutcomes,newmynoutcomes)
+				momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata)
+				oneiteration <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns)
 				currll <- oneiteration$logl
 				if (verbose) cat("current ll",currll,"\n")		
 			}
-			momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes)
+			momentdata <- calcrandom(classx,outcomex,lambdacoef,momentdata)
 			# when the ll starts decreasing, give up
 			if (currll < maxll) break()
 			maxll <- currll
@@ -221,10 +188,8 @@
 		momentdata <- lastmomentdata
  	}
  	else lambdacoef <- initlambdacoef
-
- 	#lambdacoef <- ifelse(lambdacoef < -3,-3,lambdacoef)
   	   
- 	myfit <- adaptivefit(classx,outcomex,lambdacoef,momentdata,calcSE,gh,newmyoutcomes,newmynoutcomes)
+ 	myfit <- adaptivefit(classx,outcomex,lambdacoef,calcSE,momentdata,gh,patterns)
 
 	optim.fit <- myfit$nlm
 	momentdata <- myfit$momentdata
@@ -235,8 +200,7 @@
 # transform using logistic to probabilities     
     lambdacoef <- optim.fit$estimate[(nlevel1*nclass+nclass):(nlevel1*nclass+nclass+blocksize-1)]
 	
-	final <- calclikelihood(classx,outcomex,lambdacoef,
-			momentdata,gh,newmyoutcomes,newmynoutcomes)
+	final <- calclikelihood(classx,outcomex,lambdacoef,momentdata,gh,patterns,calcfitted=TRUE)
 			
 	fitted <- final$fitted
 	classprob <- final$classprob
@@ -258,7 +222,7 @@
 
 # determine random effects
 
-	ranef <- calcrandom(classx,outcomex,lambdacoef,momentdata,newmyoutcomes,newmynoutcomes)
+	ranef <- calcrandom(classx,outcomex,lambdacoef,momentdata)
 
 
 	
